<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Dec 3, 2019
 */

/**
 * Nodeworx Cloudlinux Controller.
 *
 * @package    InterWorx
 * @subpackage Ctrl
 */
class Ctrl_Nodeworx_Cloudlinux extends Ctrl_Nodeworx_Plugin
{

    /**
     * Init.
     */
    protected function _init()
    {
    }

    /**
     * Launch action.
     */
    public function panelInfoAction()
    {
        $hostname  = Ini::get('iworx', 'hostname');
        $login_url = "https://{$hostname}:2443/nodeworx";
        $data = $this->_getPlugin()->getPanelInfo($login_url);
        $Payload = new Payload($data);
        $this->_displayPayload($Payload);
    }

    /**
     * Launch action.
     */
    public function userPanelInfoAction()
    {
        $hostname  = Ini::get('iworx', 'hostname');
        $login_url = "https://{$hostname}:2443/siteworx";
        $data = $this->_getPlugin()->getPanelInfo($login_url);
        $Payload = new Payload($data);
        $this->_displayPayload($Payload);
    }

    /**
     * dbinfo action
     */
    public function dbInfoAction()
    {
        $data = $this->_getPlugin()->getDbInfo();
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Packages script.
     *
     * @param Form_NW_Cloudlinux_Packages $Form
     */
    public function packagesCommitAction(Form_NW_Cloudlinux_Packages $Form)
    {
        if ($Form->getValue('owner')) {
            $owner = $Form->getValue('owner');
        }

        $data = $this->_getPlugin()->getPackages($owner);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Users script.
     *
     * @param Form_NW_Cloudlinux_Users $Form
     */
    public function usersCommitAction(Form_NW_Cloudlinux_Users $Form)
    {
        if ($Form->getValue('owner')) {
            $filter_owner = $Form->getValue('owner');
        }

        if ($Form->getValue('package-name')) {
            $filter_package_name = $Form->getValue('package-name');
        }

        if ($Form->getValue('package-owner')) {
            $filter_package_owner = $Form->getValue('package-owner');
        }

        if ($Form->getValue('username')) {
            $filter_username = $Form->getValue('username');
        }

        if ($Form->getValue('unix-id')) {
            $filter_unix_id = $Form->getValue('unix-id');
        }

        if ($Form->getValue('fields')) {
            $filter_fields = implode(',', $Form->getValue('fields'));
        }

        $data = $this->_getPlugin()->getUsers($filter_owner, $filter_package_name, $filter_package_owner, $filter_username, $filter_unix_id, $filter_fields);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Resellers script.
     *
     * @param Form_NW_Cloudlinux_Resellers $Form
     */
    public function resellersCommitAction(Form_NW_Cloudlinux_Resellers $Form)
    {
        if ($Form->getValue('id')) {
            $specified_id = $Form->getValue('id');
        }

        if ($Form->getValue('name')) {
            $specified_name = $Form->getValue('name');
        }

        $data = $this->_getPlugin()->getResellers($specified_id, $specified_name);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Resellers script.
     *
     */
    public function dbUserMapCommitAction()
    {
        if ($this->isRoot() !== true) {
            $this->runAsRootAndHandleReply('index', null, true);
            return;
        }

        $data = '';
        $file = '/etc/container/dbuser-map';
        IWorxFileSys::smartBarf($file, $data, 'root', 'iworx', 0664);
        $Mapper = SiteworxDbMapper::getInstance();
        $accounts = $Mapper->getAllUsers();
        foreach ($accounts as $user => $account_info) {
            $data.= "{$user} {$account_info['unix_user']} {$account_info['unix_id']}" . PHP_EOL;
        }
        IWorxFileSys::smartBarf('/etc/container/dbuser-map', $data, 'root', 'iworx', 0664);
        $this->_displaySuccess('index', 'Success');
    }

    /**
     * Admins script.
     *
     * @param Form_NW_Cloudlinux_Admins $Form
     */
    public function adminsCommitAction(Form_NW_Cloudlinux_Admins $Form)
    {
        if ($Form->getValue('is-main')) {
            $is_main = $Form->getValue('is-main');
        }

        if ($Form->getValue('name')) {
            $specified_name = $Form->getValue('name');
        }

        $data = $this->_getPlugin()->getAdmins($is_main, $specified_name);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Domains script.
     *
     * @param Form_NW_Cloudlinux_Domains $Form
     */
    public function domainsCommitAction(Form_NW_Cloudlinux_Domains $Form)
    {
        if ($Form->getValue('owner')) {
            $specified_owner = $Form->getValue('owner');
        }

        if ($Form->getValue('name')) {
            $specified_domain = $Form->getValue('name');
        }
        $data = $this->_getPlugin()->getDomains($specified_owner, $specified_domain);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Ui User Info Script.
     *
     * @param Form_NW_Cloudlinux_UserInfo $Form
     */
    public function uiUserInfoCommitAction(Form_NW_Cloudlinux_UserInfo $Form)
    {
        $token_param = $Form->getValue('token');
        $data = $this->_getPlugin()->getUiUserInfo($token_param);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Index.
     *
     * @disabledIn ~IW::WEB
     */
    public function indexAction()
    {
        $this->getView()->assign('title', 'Cloudlinux');
        $this->getView()->assign('template', 'admin');
    }
}
