<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Dec 3, 2019
 */

/**
 * Siteworx Cloudlinux php selector Controller.
 *
 * @package    InterWorx
 * @subpackage Ctrl
 */
class Ctrl_Siteworx_CloudlinuxPhp extends Ctrl_Siteworx_Plugin
{

    /**
     * Init.
     */
    protected function _init()
    {
    }

    /**
     * Panel info script.
     */
    public function panelInfoAction()
    {
        $hostname  = Ini::get('iworx', 'hostname');
        $login_url = "https://{$hostname}:2443/siteworx";
        $data = $this->_getPlugin()->getPanelInfo($login_url);
        $Payload = new Payload($data);
        $this->_displayPayload($Payload);
    }

    /**
     * Domains script.
     *
     * @param Form_SW_Cloudlinux_Domains $Form
     */
    public function domainsCommitAction(Form_SW_Cloudlinux_Domains $Form)
    {
        $specified_owner = IW::SW()->getUniqname();

        if ($Form->getValue('name')) {
            $specified_domain = $Form->getValue('name');
        }

        $data = $this->_getPlugin()->getDomains($specified_owner, $specified_domain);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Ui User Info Script.
     *
     * @param Form_NW_Cloudlinux_UserInfo $Form
     */
    public function uiUserInfoCommitAction(Form_NW_Cloudlinux_UserInfo $Form)
    {
        $token_param = $Form->getValue('token');
        $data = $this->_getPlugin()->getUiUserInfo($token_param);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Index.
     *
     * @disabledIn ~IW::WEB
     */
    public function indexAction()
    {
        $this->getView()->assign('title', 'Cloudlinux PHP Selector');
        $this->getView()->assign('template', 'php_selector');
    }
}
