<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Dec 03, 2019
 */

/**
 * Cloudlinux plugin.
 *
 * Enables Cloudlinux integration.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_Cloudlinux extends Plugin
{

    private $_usingCageFs = null;
    private $_usingNodejs = null;
    private $_usingPython = null;

    /**
     * Init.
     */
    protected function _init()
    {
        parent::_init();
        if (is_dir('/home/interworx/plugins/lvemanager')) {
            exec('rm -rf /home/interworx/plugins/lvemanager 2>/dev/null', $result, $retval);
        }
    }

    /**
     * Pre action.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function preAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:deleteCommit') {
            $domains = $params->getValue('domain');
            if (!is_array($domains)) {
                $domains = array($domains);
            }
            foreach ($domains as $domain) {
                $SW       = new SiteWorx($domain);
                $uniqname = $SW->getUniqname();

                $Cmd = new Command();
                $Cmd->runAsRoot();
                $Cmd->setCmd('/usr/share/cloudlinux/hooks/pre_modify_user.py delete --username ' .
                             $uniqname);
                $Cmd->exec();
            }
        }
        return;
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:addCommit') {
            if ($Ctrl->isViewSuccess()) {
                $user  = $params->getValue('uniqname');
                $owner = $params->getValue('reseller');
                $NW    = new NodeWorx($owner);
                $email = $NW->toAssocArray()['masteruser']['email'];

                $Cmd = new Command();
                $Cmd->runAsRoot();
                $Cmd->setCmd('/usr/share/cloudlinux/hooks/post_modify_user.py create --username ' .
                             $user . ' --owner ' . $email);
                $Cmd->exec();
            }
        }

        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:editCommit') {
            if ($Ctrl->isViewSuccess() && $params->hasInput('reseller')) {
                $user  = $params->getValue('uniqname');
                $owner = $params->getValue('reseller');
                $NW    = new NodeWorx($owner);
                $email = $NW->toAssocArray()['masteruser']['email'];

                $Cmd = new Command();
                $Cmd->runAsRoot();
                $Cmd->setCmd('/usr/share/cloudlinux/hooks/post_modify_user.py modify --username ' .
                             $user . ' --new-owner ' . $email);
                $Cmd->exec();
            }
        }

        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:deleteCommit') {
            $domains   = $params->getValue('domain');

            if (!is_array($domains)) {
                $domains = array( $domains );
            }
            foreach ($domains as $domain) {
                $SW       = new SiteWorx($domain);
                $uniqname = $SW->getUniqname();

                $Cmd = new Command();
                $Cmd->runAsRoot();
                $Cmd->setCmd('/usr/share/cloudlinux/hooks/post_modify_user.py delete --username ' .
                             $uniqname);
                $Cmd->exec();
            }
        }

        if ($ctrl_act === 'Ctrl_Siteworx_MysqlDb:addCommit') {
            IW::FC()->routeFromPHP('Ctrl_Nodeworx_Cloudlinux', 'dbUserMap', [], IW::SW()->getParentNodeworx());
        }

        if ($ctrl_act === 'Ctrl_Siteworx_MysqlUser:addCommit') {
            IW::FC()->routeFromPHP('Ctrl_Nodeworx_Cloudlinux', 'dbUserMap', [], IW::SW()->getParentNodeworx());
        }

        if ($ctrl_act === 'Ctrl_Siteworx_MysqlUser:deleteCommit') {
            IW::FC()->routeFromPHP('Ctrl_Nodeworx_Cloudlinux', 'dbUserMap', [], IW::SW()->getParentNodeworx());
        }
        return;
    }

    /**
     * Initialize Forms we care about.
     *
     * @param Form $Form
     */
    public function initializeForm(Form $Form)
    {
        $form_class = get_class($Form);

        if (in_array($form_class, [
            'Form_SW_DomainsPhp_Edit',
            'Form_SW_DomainsSlave_Edit',
            'Form_SW_DomainsSlave_Add',
            ])) {
            if (!$this->isUsingCageFs(IW::SW()->getUniqname())) {
                return;
            }
            if (PHP::getPhpMode() !== PHP::MODE_MOD_SUPHP) {
                return;
            }
            $php_version = $this->getCageFsPhpVersion(IW::SW()->getUniqname());
            if ($php_version === null) {
                return;
            }
            if ($Form->getInput('php_version') === null) {
                return;
            }
            /**
             * @var DataSource_Array $DS
             */
            $DS = $Form->getInput('php_version')->getDataSource();
            if ($DS === null) {
                return;
            }
            $DS->removeValue('system-php');
            $DS->addValueFirst('system-php', "PHP {$php_version} (Cloudlinux)");
            $Form->getInput('php_version')->setDataSource($DS);
        }
    }

    /**
     * Initialize Payload.
     *
     * @param Payload $Payload
     */
    public function initializePayload(Payload $Payload)
    {
        if (!in_array($Payload->getTag(), ['SW_Domain:slaves', 'SW_Domain:phpVersions'])) {
            return;
        }

        // Fallback to the Payload data if there is no current session SW object
        try {
            $uniqname = IW::SW()->getUniqname();
        } catch (IWorx_Exception_NotLoggedIn $e) {
            $uniqname = $this->_getUniqnameFromPayload($Payload);
        }

        // THIS ONE HERE.
        if (!$this->isUsingCageFs($uniqname)) {
            return;
        }
        if (PHP::getPhpMode() !== PHP::MODE_MOD_SUPHP) {
            return;
        }

        $php_version = $this->getCageFsPhpVersion($uniqname);
        if ($php_version === null) {
            return;
        }

        foreach ($Payload->getData() as &$row) {
            if ($row->php_is_system) {
                $row->php_version = $php_version;
            }
        }
    }

    /**
     * Get SW Uniqname from domain in Payload data
     *
     * @param Payload $Payload
     * @return string
     */
    private function _getUniqnameFromPayload(Payload $Payload)
    {
        $data = $Payload->getData();
        foreach ($data as $domainInfo) {
            $domain = $domainInfo->domain;
            return IW::NW()->getSiteWorx($domain)->getUniqname();
        }
    }

    /**
     * Integrate into the nodeworx menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        $this->_addNodeworxCloudLinuxMenu($MenuMan);
        $this->_addNodeworxLveMenuItem($MenuMan);
    }

    /**
     * Add nodeworx cloudlinux menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addNodeworxCloudLinuxMenu(IWorxMenuManager $MenuMan)
    {
        $menu_data = array(
            'text'                  => 'CloudLinux',
            'url'                   => '#',
            'class'                 => 'iw-i-settings',
            'disabled_for_reseller' => '0'
        );

        $MenuMan->addMenuItemAfter(
            'iw-menu-server',
            'cloudlinux',
            $menu_data
        );
    }

    /**
     * Add ndoeworx lve menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addNodeworxLveMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'                  => 'Lve Manager',
            'url'                   => '/nodeworx/cloudlinux',
            'class'                 => 'iw-i-plugin',
            'parent'                => 'cloudlinux',
            'disabled_for_reseller' => '0'
        );

        $MenuMan->addMenuItemAfter(
            'cloudlinux',
            'lvemanager',
            $new_data
        );
    }

    /**
     * Integrate into the siteworx menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateSiteworxMenu(IWorxMenuManager $MenuMan)
    {
        $this->_addSiteworxCloudLinuxMenu($MenuMan);
        $this->_addSiteworxApiMenuItem($MenuMan);
        $this->_addSiteworxPhpMenuItem($MenuMan);
        $this->_addSiteworxNodeMenuItem($MenuMan);
        $this->_addSiteworxPythonMenuItem($MenuMan);
        $this->_addSiteworxResourceMenuItem($MenuMan);
    }

    /**
     * Add siteworx cloudlinux menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxCloudLinuxMenu(IWorxMenuManager $MenuMan)
    {
        $menu_data = array(
            'text'  => 'CloudLinux',
            'url'   => '#',
            'class' => 'iw-i-settings',
        );

        $MenuMan->addMenuItemAfter(
            'iw-menu-pref-group',
            'cloudlinux',
            $menu_data
        );
    }

    /**
     * Add siteworx api menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxApiMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'   => 'Cloudlinux API',
            'url'    => '/siteworx/cloudlinux',
            'parent' => 'cloudlinux',
            'class'  => 'iw-i-plugin',
            'hide'   => 'WEB'
        );

        $MenuMan->addMenuItemAfter(
            'cloudlinux',
            'cloudlinux_api',
            $new_data
        );
    }

    /**
     * Add siteworx php menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxPhpMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'   => 'Php Selector',
            'url'    => '/siteworx/cloudlinux/php',
            'parent' => 'cloudlinux',
            'class'  => 'iw-i-plugin'
        );

        if (!$this->isUsingCageFs(IW::SW()->getUniqname()) || PHP::getPhpMode() !== PHP::MODE_MOD_SUPHP) {
            $new_data['hide'] = 'WEB';
        }
        if (PHP::getPhpMode() !== PHP::MODE_MOD_SUPHP) {
            $new_data['hide'] = 'WEB';
        }

        $MenuMan->addMenuItemAfter(
            'cloudlinux',
            'php_selector',
            $new_data
        );
    }

    /**
     * Add siteworx node menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxNodeMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'   => 'Node.js Selector',
            'url'    => '/siteworx/cloudlinux/nodejs',
            'parent' => 'cloudlinux',
            'class'  => 'iw-i-plugin'
        );

        if (!$this->isUsingCageFs(IW::SW()->getUniqname()) || !$this->isNodejsEnabled()) {
            $new_data['hide'] = 'WEB';
        }

        $MenuMan->addMenuItemAfter(
            'php_selector',
            'nodejs_selector',
            $new_data
        );
    }

    /**
     * Add siteworx python menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxPythonMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'   => 'Python Selector',
            'url'    => '/siteworx/cloudlinux/python',
            'parent' => 'cloudlinux',
            'class'  => 'iw-i-plugin'
        );

        if (!$this->isUsingCageFs(IW::SW()->getUniqname()) || !$this->isPythonEnabled()) {
            $new_data['hide'] = 'WEB';
        }

        $MenuMan->addMenuItemAfter(
            'nodejs_selector',
            'python_selector',
            $new_data
        );
    }

    /**
     * Add siteworx resource menu item.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxResourceMenuItem(IWorxMenuManager $MenuMan)
    {
        $new_data = array(
            'text'   => 'Resource usage',
            'url'    => '/siteworx/cloudlinux/resource',
            'parent' => 'cloudlinux',
            'class'  => 'iw-i-stats'
        );

        if (!$this->isUsingCageFs(IW::SW()->getUniqname())) {
            $new_data['hide'] = 'WEB';
        }

        $MenuMan->addMenuItemAfter(
            'python_selector',
            'resource-usage',
            $new_data
        );
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runAdmin($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/widget.php');
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runSendRequest($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/send-request.php');
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runSendRequestPhpSelector($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/send-request-php-selector.php');
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runSendRequestNodeJs($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/send-request-nodejs.php');
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runSendRequestPython($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/send-request-python.php');
    }

    /**
     * Executes the admin interface.
     *
     * @param string $hint
     */
    public function runSendRequestResource($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/send-request-resource-usage.php');
    }

    /**
     * Executes the open interface.
     *
     * @param string $hint
     */
    public function runOpen($hint)
    {
        $_GET['token'] = '--token=' . session_id() . ':' . $hint;
        session_write_close();
        umask(0022);
        include('/usr/local/interworx/plugins/cloudlinux/lib/open.php');
    }

    /**
     * Get panel info.
     *
     * @param string $login_url
     */
    public function getPanelInfo($login_url)
    {
        $name    = 'InterWorx Control Panel';
        $version = Ini::get('iworx', 'rpm.release');

        $data = [
          'name'           => $name,
          'version'        => $version,
          'user_login_url' => $login_url
        ];

        $metadata = ['result' => 'ok'];

        return ['data' => $data, 'metadata' => $metadata];
    }

    /**
     * Get domains.
     *
     * @param string $specified_owner
     * @param string $specified_domain
     * @return array
     */
    public function getDomains($specified_owner, $specified_domain)
    {
        $data = [];
        $VhostReader = new ApacheVhostReader();
        $master_domains = $VhostReader->getSiteWorxDomainList();

        foreach ($master_domains as $master_domain) {
            $SW      = new SiteWorx($master_domain);
            $domains = $SW->getDomainNameList();
            foreach ($domains as $domain) {
                //do filtering.
                $Domain = new Domain($domain);
                if ($Domain->isPointer() || $Domain->getType() == 'redirect') {
                    continue;
                }
                if (!empty($specified_owner) && $SW->getUniqname() !== $specified_owner) {
                    continue;
                }
                if (!empty($specified_domain) && $domain !== $specified_domain) {
                    continue;
                }

                $is_master = ($domain === $SW->getMasterDomainName() ? true : false);

                $data['data'][$domain] = [
                  'owner'         => $SW->getUniqname(),
                  'document_root' => $SW->getWebRoot($domain),
                  'is_main'       => $is_master
                ];
            }
        }
        $data['metadata'] = ['result' => 'ok'];
        return $data;
    }

    /**
     * Get UI User Info.
     *
     * @param string $token_param
     * @return array
     */
    public function getUiUserInfo($token_param)
    {
        // Seed the session ID so we get the right one.
        session_write_close();
        list($sessionId, $hint) = explode(':', $token_param);
        session_id($sessionId);
        session_name('interworx-cp');
        session_start();

        if ($hint === 'admin') {
            $NW = new NodeWorx_User($_SESSION['s_nodeworx_user_id']);
            $userName  = $NW->toAssocArray()['email'];
            $userId    = $_SESSION['s_nodeworx_user_id'] + 50000;
            $userType  = 'admin';
            $baseUri   = '/nodeworx/cloudlinux';
            $assetsUri = '/plugins/cloudlinux';
        } elseif ($hint === 'reseller') {
            $NW = new NodeWorx_User($_SESSION['s_nodeworx_user_id']);
            $userName  = $NW->toAssocArray()['email'];
            $userId    = $_SESSION['s_nodeworx_user_id'] + 50000;
            $userType  = 'reseller';
            $baseUri   = '/nodeworx/cloudlinux';
            $assetsUri = '/plugins/cloudlinux';
        } else {
            $SW = new SiteWorx($_SESSION['s_siteworx_domain']);
            $userName  = $SW->getUniqname();
            $userId    = $SW->getUID();
            $userType  = 'user';
            $baseUri   = '/siteworx/cloudlinux';
            $assetsUri = '/plugins/cloudlinux';
            $userDomain = $SW->getWorkingDomainName();
        }

        $lang = 'en';

        $data = [
          'userName'  => $userName,
          'userId'    => $userId,
          'userType'  => $userType,
          'baseUri'   => $baseUri,
          'assetsUri' => $assetsUri,
          'lang'      => $lang
        ];

        if (isset($userDomain)) {
            $data['userDomain'] = $userDomain;
        }

        return $data;
    }

    /**
     * Get Users.
     *
     * @param string $filter_owner
     * @param string $filter_package_name
     * @param string $filter_package_owner
     * @param string $filter_username
     * @param string $filter_unix_id
     * @param string $filter_fields
     * @return array
     */
    public function getUsers($filter_owner, $filter_package_name, $filter_package_owner, $filter_username, $filter_unix_id, $filter_fields)
    {
        $users = ShellAccount::getSiteworxUsers();
        $data  = [];

        foreach ($users as $user) {
            $uid = $user['uid'];
            $username = $user['user'];
            $SW = new SiteWorx($user['siteworx']);
            $email = $SW->getMasterUser()->getEmail();
            $domain = $SW->getMasterDomainName();
            $locale_code = 'EN_us';
            $owner = $SW->getParentMasterUser()->getEmail();
            $package_name = $SW->findPackageTemplateName();

            if (!empty($filter_owner) && $filter_owner !== $owner) {
                continue;
            }

            if (!empty($filter_package_name) && !empty($filter_package_owner) &&
                ($filter_package_name !== $package_name || $filter_package_owner !== $owner)) {
                continue;
            }

            if (!empty($filter_username) && $filter_username !== $username) {
                continue;
            }

            if (!empty($filter_unix_id) && $filter_unix_id !== $uid) {
                continue;
            }

            $data['data'][] = ['id' => intval($uid),
                               'username' => $username,
                               'owner' => $owner,
                               'domain' => $domain,
                               'package' => [
                                 'name' => $package_name,
                                 'owner' => $owner
                               ],
                               'email' => $email,
                               'locale_code' => $locale_code
            ];
        }

        if (is_null($data['data'])) {
            $data['data'] = [];
        }

        if (!empty($filter_fields)) {
            $filter_fields = explode(',', $filter_fields);
            foreach ($data['data'] as $key => $user) {
                //magic
                $filtered_data[$key] = array_filter(
                    $user,
                    function ($key) use ($filter_fields) {
                        return in_array($key, $filter_fields);
                    },
                    ARRAY_FILTER_USE_KEY
                );
            }
            $data['data'] = $filtered_data;
        }

        $data['metadata'] = ['result' => 'ok'];

        if (is_null($data['data'])) {
            $data['data'] = [];
        }

        return $data;
    }

    /**
     * Get Resellers.
     *
     * @param integer $specified_id
     * @param string  $specified_name
     */
    public function getResellers($specified_id, $specified_name)
    {
        $NodeWorx = new NodeWorx(NodeWorx::MASTER_ID);
        $reseller_list = $NodeWorx->getResellers($with_master_reseller = true);
        $data = [];

        foreach ($reseller_list as $reseller) {
            $reseller = $reseller['masteruser'];

            if ($reseller['userid'] === 1) {
                continue;
            }

            //do filtering.
            if (!empty($specified_name) && $reseller['email'] !== $specified_name) {
                continue;
            }
            if (!empty($specified_id) && $reseller['userid'] != $specified_id) {
                continue;
            }

            $data['data'][] = ['name' => $reseller['email'],
                               'locale_code' => 'EN_us',
                               'email' => $reseller['email'],
                               'id' => $reseller['userid'] + 50000];
        }

        $data['metadata'] = ['result' => 'ok'];

        if (is_null($data['data'])) {
            $data['data'] = [];
        }

        return $data;
    }

    /**
     * Get Admins.
     *
     * @param string $is_main
     * @param string $specified_name
     * @return array
     */
    public function getAdmins($is_main, $specified_name)
    {
        $NW = new NodeWorx(Nodeworx::MASTER_ID);
        $master_user = $NW->getUserList('master'){0}->toAssocArray();
        $data = [];


        $data['data'][] = ['name' => $master_user['email'],
                           'unix_user' => null,
                           'locale_code' => 'EN_us',
                           'email' => $master_user['email'],
                           'is_main' => $master_user['userid'] == 1 ? true : false];


        $data['metadata'] = ['result' => 'ok'];
        if (is_null($data['data'])) {
            $data['data'] = [];
        }
        return $data;
    }

    /**
     * Get Packages.
     *
     * @param string $owner
     * @return array
     */
    public function getPackages($owner)
    {
        $data = [];
        if (!empty($owner)) {
            $User = new NodeWorx($owner);
            $packages = $User->getPackageNames();
            foreach ($packages as $package) {
                $data['data'][] = ['name' => $package,
                                   'owner' => $owner
                ];
            }
        } else {
            $NW = new NodeWorx(NodeWorx::MASTER_ID);
            $email = $NW->getUserInfoById(NodeWorx::MASTER_ID)['email'];
            $admin_packages = $NW->getPackageNames();

            foreach ($admin_packages as $package) {
                $data['data'][] = ['name' => $package,
                                   'owner' => $email
                ];
            }

            $resellers = $NW->getResellers();
            foreach ($resellers as $reseller) {
                $email = $reseller['masteruser']['email'];
                $ResellerNW = new NodeWorx($email);
                $packages = $ResellerNW->getPackageNames();
                foreach ($packages as $package) {
                    $data['data'][] = ['name' => $package,
                                       'owner' => $email
                    ];
                }
            }
        }
        $data['metadata'] = ['result' => 'ok'];
        return $data;
    }

    /**
     * Get DB Info.
     *
     * @return array
     */
    public function getDbInfo()
    {
        if (IW::Env()->isNode()) {
            $dsn = Ini::get(Ini::MYSQL, 'rootdsn.orig');
        } else {
            $dsn = Ini::get(Ini::MYSQL, 'rootdsn');
        }
        $dsninfo = DB::parseDSN($dsn);
        $host = 'localhost';

        //this is dumb, but didn't want to hard code it.
        $mysql_port              = MySQL::getPortNumbers()[0];
        $data                    = [];
        $data['data']['mysql']['access'] = [
          'login'    => $dsninfo['username'],
          'password' => $dsninfo['password'],
          'host'     => $host,
          'port'     => $mysql_port
        ];

        $data['data']['mysql']['mapping'] = new stdClass;
        foreach (SiteWorx::getValidAccounts() as $SW) {
            $unix_user = $SW->getUniqname();
            $databases = SiteworxDbMapper::getInstance()->getUsers($SW);

            if (empty($databases)) {
                continue;
            }

            $data['data']['mysql']['mapping']->$unix_user = $databases;
        }
        $data['metadata'] = ['result' => 'ok'];
        return $data;
    }

    /**
     * Is Using CageFS.
     *
     * @param string $unixuser
     * @return boolean
     */
    public function isUsingCageFs($unixuser)
    {
        //@todo sanity check $unixuser
        if ($this->_usingCageFs === null) {
            $cmd = '/usr/bin/selectorctl --user-current --user=' . $unixuser;
            $Cmd = new Command($cmd);
            $Cmd->exec();
            $this->_usingCageFs = ($Cmd->getRetval() == '0');
        }
        return $this->_usingCageFs;
    }
    /**
     * Is NodejsEnabled.
     *
     * @return boolean
     */
    public function isNodejsEnabled()
    {
        if ($this->_usingNodejs === null) {
            $conf = '/etc/cl.nodejs/selector.json';
            if (is_readable($conf)) {
                $this->_usingNodejs = (json_decode(file_get_contents($conf))->selector_enabled);
            } else {
                $cmd = '/usr/sbin/cloudlinux-selector --get-selector-status --json --interpreter nodejs';
                $Cmd = new Command($cmd);
                $Cmd->runAsRoot();
                $Cmd->exec();

                $result = json_decode($Cmd->getOutput()[0]);
                $this->_usingNodejs = ($result->selector_enabled == true);
            }
        }
        return $this->_usingNodejs;
    }

    /**
     * Is NodejsEnabled.
     *
     * @return boolean
     */
    public function isPythonEnabled()
    {
        if ($this->_usingPython === null) {
            $conf = '/etc/cl.python/selector.json';
            if (is_readable($conf)) {
                $this->_usingPython = (json_decode(file_get_contents($conf))->selector_enabled);
            } else {
                $cmd = '/usr/sbin/cloudlinux-selector --get-selector-status --json --interpreter python';
                $Cmd = new Command($cmd);
                $Cmd->runAsRoot();
                $Cmd->exec();
                $result = json_decode($Cmd->getOutput()[0]);
                $this->_usingPython = ($result->selector_enabled == true);
            }
        }
        return $this->_usingPython;
    }

    /**
     * Get CageFS Php Version.
     *
     * @param string $unixuser
     * @return string
     */
    public function getCageFsPhpVersion($unixuser)
    {
        //@todo sanity check $unixuser
        $cmd = '/usr/bin/selectorctl --user-current --user=' . $unixuser;
        $Cmd = new Command($cmd);
        $Cmd->exec();
        if ($Cmd->getRetval() != '0') {
            return null;
        }
        $output = $Cmd->getOutput();
        if (!isset($output[0])) {
            return null;
        }
        $data = explode("\t", $output[0]);
        if (!isset($data[1])) {
            return null;
        }

        return $data[1];
    }

    /**
     * The nodeworx log location for cl.
     *
     * @return string
     */
    public function getLogFile()
    {
        return "/home/interworx/var/log/cloudlinux.log";
    }
}
